// -*- C++ -*-
/*!
 * @file  DIOInterfaceModuleRTC.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "DIOInterfaceModuleRTC.h"

// Module specification
// <rtc-template block="module_spec">
static const char* diointerfacemodulertc_spec[] =
  {
    "implementation_id", "DIOInterfaceModuleRTC",
    "type_name",         "DIOInterfaceModuleRTC",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "Saitama Univ. Design Lab.",
    "category",          "EngineeringSample",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
DIOInterfaceModuleRTC::DIOInterfaceModuleRTC(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_digitalInputIn("digitalInput", m_digitalInput),
    m_digitalOutputOut("digitalOutput", m_digitalOutput)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
DIOInterfaceModuleRTC::~DIOInterfaceModuleRTC()
{
}



RTC::ReturnCode_t DIOInterfaceModuleRTC::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("digitalInput", m_digitalInputIn);
  
  // Set OutPort buffer
  addOutPort("digitalOutput", m_digitalOutputOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>
  m_digitalOutput.data.length(32);
  m_fromDIOValueOld = 4294967295;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DIOInterfaceModuleRTC::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DIOInterfaceModuleRTC::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DIOInterfaceModuleRTC::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t DIOInterfaceModuleRTC::onActivated(RTC::UniqueId ec_id)
{
	std::cout << "onActivatedJn" << std::endl;

	m_dll = LoadLibrary("fbidio.dll");
	if( m_dll == NULL ){
		std::cout << "G[: fbidio.dllǍs" << std::endl;
		RTC_ERROR(("G[: fbidio.dllǍs"));
		return RTC::RTC_ERROR;
	}else{
		std::cout << "fbidio.dllǍ" << std::endl;
		RTC_INFO(("fbidio.dllǍ"));
		DioOpen = (dll_DioOpen)GetProcAddress(m_dll,"DioOpen");
		DioClose = (dll_DioClose)GetProcAddress(m_dll,"DioClose");
		DioInputDword = (dll_DioInputDword)GetProcAddress(m_dll,"DioInputDword");
		DioOutputDword = (dll_DioOutputDword)GetProcAddress(m_dll,"DioOutputDword");
		if( (DioOpen == NULL) || (DioClose == NULL) || (DioInputDword == NULL) || (DioOutputDword == NULL) ){
			std::cout << "G[: DioOpenCDioCloseCDioInputDwordCDioOutputDwordǍs" << std::endl;
			RTC_ERROR(("G[: DioOpenCDioCloseCDioInputDwordCDioOutputDwordǍs"));
			return RTC::RTC_ERROR;
		}else{
			std::cout << "DioOpenCDioCloseCDioInputDwordCDioOutputDwordǍ" << std::endl;
			RTC_INFO(("DioOpenCDioCloseCDioInputDwordCDioOutputDwordǍ"));
		}
	}

	m_DeviceHandle = DioOpen("FBIDIO1",0x0000);
	if(m_DeviceHandle == INVALID_HANDLE_VALUE){
		std::cout << "G[: DioOpens" << std::endl;
		RTC_ERROR(("G[: DioOpens"));
		return RTC::RTC_ERROR;
	}else{
		std::cout << "DioOpen" << std::endl;
		RTC_INFO(("DioOpen"));
		
		DWORD toDIOValue = 0;
		m_Ret = DioOutputDword(m_DeviceHandle, 0, toDIOValue);
		if(m_Ret){
			std::cout << "G[: ߂l " << m_Ret << " (10i)ł" << std::endl;
			std::cout << "DioOutputDwords" << std::endl;
			RTC_ERROR(("G[: ߂l %d (10i)ł",m_Ret));
			RTC_ERROR(("DioOutputDwords"));
			return RTC::RTC_ERROR;
		}else{
			std::cout << "(10i)toDIOValue =   " << toDIOValue << std::endl;
			std::cout.setf( std::ios::hex, std::ios::basefield );
			std::cout.setf( std::ios::showbase );
			std::cout << "(16i)toDIOValue =   " << toDIOValue << std::endl;
			std::cout.setf( std::ios::dec, std::ios::basefield );
			std::cout << "( 2i)toDIOValue =   " << std::bitset<32>((unsigned long long)toDIOValue) << std::endl;
			RTC_INFO(("( 2i)toDIOValue = %ld",std::bitset<32>((unsigned long long)toDIOValue)));
		}
		
		for(int i=0;i<m_digitalOutput.data.length();i++){
			m_digitalOutput.data[i] = false;
		}
		m_digitalOutputOut.write();
	}

	std::cout << "onActivatedI" << std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t DIOInterfaceModuleRTC::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout << "onDeactivatedJn" << std::endl;
	
	if(m_DeviceHandle != INVALID_HANDLE_VALUE){
		for(int i=0;i<m_digitalOutput.data.length();i++){
			m_digitalOutput.data[i] = false;
		}
		m_digitalOutputOut.write();
		
		DWORD toDIOValue = 0;
		m_Ret = DioOutputDword(m_DeviceHandle, 0, toDIOValue);
		if(m_Ret){
			std::cout << "G[: ߂l " << m_Ret << " (10i)ł" << std::endl;
			std::cout << "DioOutputDwords" << std::endl;
			RTC_ERROR(("G[: ߂l %d (10i)ł",m_Ret));
			RTC_ERROR(("DioOutputDwords"));
		}else{
			std::cout << "(10i)toDIOValue =   " << toDIOValue << std::endl;
			std::cout.setf( std::ios::hex, std::ios::basefield );
			std::cout.setf( std::ios::showbase );
			std::cout << "(16i)toDIOValue =   " << toDIOValue << std::endl;
			std::cout.setf( std::ios::dec, std::ios::basefield );
			std::cout << "( 2i)toDIOValue =   " << std::bitset<32>((unsigned long long)toDIOValue) << std::endl;
			RTC_INFO(("( 2i)toDIOValue = %ld",std::bitset<32>((unsigned long long)toDIOValue)));
		}

		m_Ret = DioClose(m_DeviceHandle);
		if(m_Ret){
			std::cout << "G[: ߂l " << m_Ret << " (10i)ł" << std::endl;
			std::cout << "DioCloses" << std::endl;
			RTC_ERROR(("G[: ߂l %d (10i)ł",m_Ret));
			RTC_ERROR(("DioCloses"));
		}else{
			std::cout << "DioClose" << std::endl;
			RTC_INFO(("DioClose"));
		}
	}
	if( (DioOpen != NULL) && (DioClose != NULL) && (DioInputDword != NULL) && (DioOutputDword != NULL) ){
		if( FreeLibrary(m_dll) ){
			std::cout << "fbidio.dll" << std::endl;
			RTC_INFO(("fbidio.dll"));
		}else{
			std::cout << "G[: fbidio.dlls" << std::endl;
			RTC_ERROR(("G[: fbidio.dlls"));
		}
	}

	std::cout << "onDeactivatedI" << std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t DIOInterfaceModuleRTC::onExecute(RTC::UniqueId ec_id)
{
	//to DIO from inport
	if(m_digitalInputIn.isNew()){
		m_digitalInputIn.read();
		DWORD toDIOValue = 0;
		for(int i=0;i<m_digitalInput.data.length();i++){
			if(m_digitalInput.data[i]){
				toDIOValue = toDIOValue + (DWORD)(pow(2.0,i));
			}
		}
		m_Ret = DioOutputDword(m_DeviceHandle, 0, toDIOValue);
		if(m_Ret){
			std::cout << "G[: ߂l " << m_Ret << " (10i)ł" << std::endl;
			std::cout << "DioOutputDwords" << std::endl;
			RTC_ERROR(("G[: ߂l %d (10i)ł",m_Ret));
			RTC_ERROR(("DioOutputDwords"));
			return RTC::RTC_ERROR;
		}else{
			std::cout << "(10i)toDIOValue =   " << toDIOValue << std::endl;
			std::cout.setf( std::ios::hex, std::ios::basefield );
			std::cout.setf( std::ios::showbase );
			std::cout << "(16i)toDIOValue =   " << toDIOValue << std::endl;
			std::cout.setf( std::ios::dec, std::ios::basefield );
			std::cout << "( 2i)toDIOValue =   " << std::bitset<32>((unsigned long long)toDIOValue) << std::endl;
			RTC_INFO(("( 2i)toDIOValue = %ld",std::bitset<32>((unsigned long long)toDIOValue)));
		}
	}

	//to outport from DIO
	DWORD fromDIOValue = 0;
	m_Ret = DioInputDword(m_DeviceHandle, 0, &fromDIOValue);
	if(m_Ret){
		std::cout << "G[: ߂l " << m_Ret << " (10i)ł" << std::endl;
		std::cout << "DioInputDwords" << std::endl;
		RTC_ERROR(("G[: ߂l %d (10i)ł",m_Ret));
			RTC_ERROR(("DioInputDwords"));
		return RTC::RTC_ERROR;
	}else{
		if(m_fromDIOValueOld != fromDIOValue){
			m_fromDIOValueOld = fromDIOValue;
			std::cout << "(10i)fromDIOValue = " << fromDIOValue << std::endl;
			std::cout.setf( std::ios::hex, std::ios::basefield );
			std::cout.setf( std::ios::showbase );
			std::cout << "(16i)fromDIOValue = " << fromDIOValue << std::endl;
			std::cout.setf( std::ios::dec, std::ios::basefield );
			std::cout << "( 2i)fromDIOValue = " << std::bitset<32>((unsigned long long)fromDIOValue) << std::endl;
			RTC_INFO(("( 2i)fromDIOValue = %ld",std::bitset<32>((unsigned long long)fromDIOValue)));
			
			for(int i=m_digitalOutput.data.length();i>0;i--){
				if( fromDIOValue >= (DWORD)(pow(2.0,i-1)) ){
					fromDIOValue = fromDIOValue - (DWORD)(pow(2.0,i-1));
					m_digitalOutput.data[i-1] = true;
				}else{
					m_digitalOutput.data[i-1] = false;
				}
			}
			m_digitalOutputOut.write();
		}
	}

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t DIOInterfaceModuleRTC::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DIOInterfaceModuleRTC::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DIOInterfaceModuleRTC::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DIOInterfaceModuleRTC::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t DIOInterfaceModuleRTC::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void DIOInterfaceModuleRTCInit(RTC::Manager* manager)
  {
    coil::Properties profile(diointerfacemodulertc_spec);
    manager->registerFactory(profile,
                             RTC::Create<DIOInterfaceModuleRTC>,
                             RTC::Delete<DIOInterfaceModuleRTC>);
  }
  
};


