// -*- C++ -*-
/*!
 * @file  IntegratedControlRTC.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "IntegratedControlRTC.h"

// Module specification
// <rtc-template block="module_spec">
static const char* integratedcontrolrtc_spec[] =
  {
    "implementation_id", "IntegratedControlRTC",
    "type_name",         "IntegratedControlRTC",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "Saitama Univ. Design Lab.",
    "category",          "Industrial Robot",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.cameraCalibrationX", "380",
    "conf.default.cameraLengthX", "380.0",
    "conf.default.cameraXLimitMax", "460",
    "conf.default.cameraXLimitMin", "210",
    "conf.default.cameraYLimitMax", "480",
    "conf.default.cameraYLimitMin", "0",
    "conf.default.cycleCount", "0",
    "conf.default.teachingData", "PointData.txt",
    // Widget
    "conf.__widget__.cameraCalibrationX", "text",
    "conf.__widget__.cameraLengthX", "text",
    "conf.__widget__.cameraXLimitMax", "text",
    "conf.__widget__.cameraXLimitMin", "text",
    "conf.__widget__.cameraYLimitMax", "text",
    "conf.__widget__.cameraYLimitMin", "text",
    "conf.__widget__.cycleCount", "text",
    "conf.__widget__.teachingData", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
IntegratedControlRTC::IntegratedControlRTC(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_digitalInputIn("digitalInput", m_digitalInput),
    m_positionIn("position", m_position),
    m_digitalOutputOut("digitalOutput", m_digitalOutput),
    m_ManipulatorCommonInterface_CommonPort("ManipulatorCommonInterface_Common"),
    m_ManipulatorCommonInterface_MiddlePort("ManipulatorCommonInterface_Middle"),
    m_triggerPort("trigger")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
IntegratedControlRTC::~IntegratedControlRTC()
{
}



RTC::ReturnCode_t IntegratedControlRTC::onInitialize()
{
	std::cout<<"Initialize"<<std::endl;

  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("digitalInput", m_digitalInputIn);
  addInPort("position", m_positionIn);
  
  // Set OutPort buffer
  addOutPort("digitalOutput", m_digitalOutputOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  m_ManipulatorCommonInterface_CommonPort.registerConsumer("ManipulatorCommonInterface_Common", "JARA_ARM::ManipulatorCommonInterface_Common", m_ManipulatorCommonInterface_Common);
  m_ManipulatorCommonInterface_MiddlePort.registerConsumer("ManipulatorCommonInterface_Middle", "JARA_ARM::ManipulatorCommonInterface_Middle", m_ManipulatorCommonInterface_Middle);
  m_triggerPort.registerConsumer("recogPort", "RecognitionService", m_RecoginitionService);
  
  // Set CORBA Service Ports
  addPort(m_ManipulatorCommonInterface_CommonPort);
  addPort(m_ManipulatorCommonInterface_MiddlePort);
  addPort(m_triggerPort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("cameraCalibrationX", m_cameraCalibrationX, "380");
  bindParameter("cameraLengthX", m_cameraLengthX, "380.0");
  bindParameter("cameraXLimitMax", m_cameraXLimitMax, "460");
  bindParameter("cameraXLimitMin", m_cameraXLimitMin, "210");
  bindParameter("cameraYLimitMax", m_cameraYLimitMax, "480");
  bindParameter("cameraYLimitMin", m_cameraYLimitMin, "0");
  bindParameter("cycleCount", m_cycleCount, "0");
  bindParameter("teachingData", m_teachingData, "PointData.txt");

  // </rtc-template>

  std::cout<<"InitializeI"<<std::endl;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t IntegratedControlRTC::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t IntegratedControlRTC::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t IntegratedControlRTC::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t IntegratedControlRTC::onActivated(RTC::UniqueId ec_id)
{
	std::cout<<"Activate"<<std::endl;

	Sleep(5000);

	//eB[`Of[^ǂݍ
	FILE * fp;
	int len = m_teachingData.length();
	char* FileName = new char[len+1];
	memcpy(FileName,m_teachingData.c_str(),len+1);
	
	fp=fopen( FileName , "r");
	if( fp == NULL ) {
		std::cout<<"t@Cǂݍ߂܂"<<std::endl;
		RTC_ERROR(("t@Cǂݍ߂܂"));
		//ErrorJ
		return RTC::RTC_ERROR;
	}
	
	int i,num;
	//i=1;
	for(i=1;i<6;i++){
		fscanf( fp, "P%d=(%lf,%lf,%lf,%lf,%lf,%lf)(%lf,%lf)\n", &num, &m_P[i][0], &m_P[i][1], &m_P[i][2], &m_P[i][3], &m_P[i][4], &m_P[i][5], &m_P[i][6], &m_P[i][7]);	
		std::cout<<"P"<<num<<"=("<<m_P[i][0]<<" , "<<m_P[i][1]<<" , "<<m_P[i][2]<<" , "<<m_P[i][3]<<" , "<<m_P[i][4]<<" , "<<m_P[i][5]<<")("<<m_P[i][6]<<","<<m_P[i][7]<<")"<<std::endl;
		RTC_INFO(("P%d=( %lf , %lf , %lf , %lf , %lf , %lf)( %lf , %lf)",num,m_P[i][0],m_P[i][1],m_P[i][2],m_P[i][3],m_P[i][4],m_P[i][5],m_P[i][6],m_P[i][7]));
		m_P[i][3]=m_P[i][3]/180.0*M_PI;
		m_P[i][4]=m_P[i][4]/180.0*M_PI;
		m_P[i][5]=m_P[i][5]/180.0*M_PI;
		//ϊsɑ
		//pX->Y->Z]ł̒l
			//1
			m_Pos[i].carPos[0][0]=cos(m_P[i][5])*cos(m_P[i][4]);
			m_Pos[i].carPos[1][0]=sin(m_P[i][5])*cos(m_P[i][4]);
			m_Pos[i].carPos[2][0]=-sin(m_P[i][4]);
			
			//2
			m_Pos[i].carPos[0][1]=-sin(m_P[i][5])*cos(m_P[i][3])+cos(m_P[i][5])*sin(m_P[i][4])*sin(m_P[i][3]);
			m_Pos[i].carPos[1][1]=cos(m_P[i][5])*cos(m_P[i][3])+sin(m_P[i][5])*sin(m_P[i][4])*sin(m_P[i][3]);
			m_Pos[i].carPos[2][1]=cos(m_P[i][4])*sin(m_P[i][3]);
			
			//3
			m_Pos[i].carPos[0][2]=sin(m_P[i][5])*sin(m_P[i][3])+cos(m_P[i][5])*sin(m_P[i][4])*cos(m_P[i][3]);
			m_Pos[i].carPos[1][2]=-cos(m_P[i][5])*sin(m_P[i][3])+sin(m_P[i][5])*sin(m_P[i][4])*cos(m_P[i][3]);
			m_Pos[i].carPos[2][2]=cos(m_P[i][4])*cos(m_P[i][3]);
			
			//4
			m_Pos[i].carPos[0][3]=m_P[i][0];
			m_Pos[i].carPos[1][3]=m_P[i][1];
			m_Pos[i].carPos[2][3]=m_P[i][2];
	
	}
	//t@CN[Y
	fclose( fp );
	
	//JԂ񐔏
	m_Repeat = 0;

	//DIO
	m_digitalOutput.data.length(3);
	m_digitalOutput.data[0]=false;
	m_digitalOutput.data[1]=false;
	m_digitalOutput.data[2]=false;
	m_digitalOutputOut.write();

	
	//\tg~bglݒ
	std::cout<<"\tg~bgln[h~bglɐݒ"<<std::endl;
	RTC_INFO(("\tg~bgln[h~bglɐݒ"));
	JARA_ARM::LimitSeq softLimit;
	softLimit.length(2);
	softLimit[0].upper=0.20;//[m]
	softLimit[0].lower=0.00;//[m]
	softLimit[1].upper=0.10;//[m]
	softLimit[1].lower=0.0;//[m]
	m_rid=m_ManipulatorCommonInterface_Common->setSoftLimitJoint(softLimit);
	if(m_rid->id != 0){//Error
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}
	
	//x[%]ݒ
	std::cout<<"ő呬xɑ΂銄30[%]ɐݒ"<<std::endl;
	RTC_INFO(("ő呬xɑ΂銄30[%]ɐݒ"));
	
	m_rid=m_ManipulatorCommonInterface_Middle->setSpeedCartesian(30);
	if(m_rid->id != 0){//Error
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}
	
	m_rid=m_ManipulatorCommonInterface_Middle->setSpeedJoint(30);
	if(m_rid->id != 0){//Error
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}
	

	//JnmF(y/n)
	char c;
	std::cout<<"_Aʒu(J1=0.0[mm],J2=0.0[mm])ɈړJn܂(y/n)"<<std::endl;
	std::cout<<"n̏ꍇRTCERRORԂɑJڂ܂"<<std::endl;
	RTC_INFO(("_Aʒu(J1=0.0[mm],J2=0.0[mm])ɈړJn܂(y/n)"));
	RTC_INFO(("n̏ꍇRTCERRORԂɑJڂ܂"));
	std::cin>>c;
	switch(c){
	case 'y':
		//T[{ON
		m_rid=m_ManipulatorCommonInterface_Common->servoON();
		if(m_rid->id != 0){//Error
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		break;
	default:
		//ErrorJ
		return RTC::RTC_ERROR;
	}

	//nhJ(DIOo̓|[g[1],[2])
	std::cout<<"nhOPEN"<<std::endl;
	RTC_INFO(("nhOPEN"));
	m_digitalOutput.data[1]=true;
	m_digitalOutput.data[2]=false;
	m_digitalOutputOut.write();
	while(1){
		if(m_digitalInputIn.isNew()){
			m_digitalInputIn.read();
			if(m_digitalInput.data[1] == true){
				break;
			}
		}
	}

	//_Aʒu(PHome)Ɉړ
	JARA_ARM::JointPos jnt;
	jnt.length(2);		
	jnt[0]=0.0;//[m]
	jnt[1]=0.0;//[m]
	m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
	if(m_rid->id != 0){
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}

	do{
		m_ManipulatorCommonInterface_Common->getState(m_state);
		if(m_state&0x4){
			std::cout <<"A["<< std::endl;
			RTC_ERROR(("A["));
			return RTC::RTC_ERROR;
		}
	}while(m_state&0x2);

	std::cout<<"ActivateI"<<std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t IntegratedControlRTC::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout<<"Deactivate"<<std::endl;

	//DIO
	m_digitalOutput.data.length(3);
	m_digitalOutput.data[0]=false;
	m_digitalOutput.data[1]=false;
	m_digitalOutput.data[2]=false;
	m_digitalOutputOut.write();

	do{
		m_ManipulatorCommonInterface_Common->getState(m_state);
		if(m_state&0x4){
			std::cout <<"A["<< std::endl;
			RTC_ERROR(("A["));
			return RTC::RTC_ERROR;
		}
	}while(m_state&0x2);

	//_Aʒu(PHome)Ɉړ
	JARA_ARM::JointPos jnt;
	jnt.length(2);		
	jnt[0]=0.0;//[m]
	jnt[1]=0.0;//[m]
	m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
	if(m_rid->id != 0){
		std::cout << std::endl<< m_rid->comment << std::endl;
		
		JARA_ARM::AlarmSeq_var alarms;
		m_ManipulatorCommonInterface_Common->getActiveAlarm(alarms);
		int len=alarms->length();
		if(len == 0){
			std::cout << "Non Alarm"<< std::endl;
			RTC_INFO(("Non Alarm"));
		}else{
			for(int i=0;i<len;i++){
				std::cout << "Alarm"<< std::endl << alarms[i].description << std::endl;
				RTC_INFO(("Alarm : %s",alarms[i].description));
			}
		}

		m_ManipulatorCommonInterface_Common->getState(m_state);
		if(m_state&0x1){
			std::cout <<"T[{ON"<< std::endl;
			RTC_INFO(("T[{ON"));
		}
		if(m_state&0x2){
			std::cout <<"쒆"<< std::endl;
			RTC_INFO(("쒆"));
		}
		if(m_state&0x4){
			std::cout <<"A["<< std::endl;
			RTC_INFO(("A["));
		}
		if(m_state&0x8){
			std::cout <<"Move߂̃obt@t"<< std::endl;
			RTC_INFO(("Move߂̃obt@t"));
		}
		if(m_state&0x10){
			std::cout <<"ꎞ~"<< std::endl;
			RTC_INFO(("ꎞ~"));
		}

		//ErrorJ
		return RTC::RTC_ERROR;
	}

	do{
		m_ManipulatorCommonInterface_Common->getState(m_state);
		if(m_state&0x4){
			std::cout <<"A["<< std::endl;
			RTC_ERROR(("A["));
			return RTC::RTC_ERROR;
		}
	}while(m_state&0x2);

	//T[{OFF
	std::cout<<"T[{OFF"<<std::endl;
	RTC_INFO(("T[{OFF"));
	m_rid=m_ManipulatorCommonInterface_Common->servoOFF();
	if(m_rid->id != 0){
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}

	std::cout<<"DeactivateI"<<std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t IntegratedControlRTC::onExecute(RTC::UniqueId ec_id)
{
	//JԂ
	if((m_cycleCount>=0)&&(m_Repeat >= m_cycleCount)){
		m_Repeat=0;
	}else{
		int CameraXMin = m_cameraXLimitMin;
		int count=0;
		
		//RxA(DIOo̓|[g[0])
		m_digitalOutput.data[0]=true;
		m_digitalOutputOut.write();
		
		//[v
		while(1){
			//USBJ烏[Nʒu̎擾
			if(m_positionIn.isNew()){
				m_positionIn.read();
			
				//
				if((m_position.data[11]>CameraXMin)&&(m_position.data[11]<m_cameraXLimitMax)&&(m_position.data[15]>m_cameraYLimitMin)&&(m_position.data[15]<m_cameraYLimitMax)){
					count++;
					CameraXMin = m_position.data[11];
					if(count == 2){
						//RxA~(DIOo̓|[g[0])
						m_digitalOutput.data[0]=false;
						m_digitalOutputOut.write();
						break;
					}
				
				}else if(m_position.data[11] != CameraXMin){

					CameraXMin = m_cameraXLimitMin;
					count = 0;

					//ԊOZT(DIO̓|[g[0])mF
					if(m_digitalInputIn.isNew()){
						m_digitalInputIn.read();
						if(m_digitalInput.data[0] == true){
							std::cout << "ԊOZTmF" << std::endl;
							std::cout << "xgRxA~AT[{OFF" << std::endl;
							RTC_ERROR(("ԊOZTmF"));
							RTC_ERROR(("xgRxA~AT[{OFF"));

							//~
							m_digitalOutput.data[0]=false;
							m_digitalOutput.data[1]=false;
							m_digitalOutput.data[2]=false;
							m_digitalOutputOut.write();
							m_ManipulatorCommonInterface_Common->servoOFF();
							//ErrorJ
							return RTC::RTC_ERROR;
						}
					}
				}
			}
		}

		count=0;
		int CameraXPosition=0;
		static int CameraXPositionBuf[7];
		//USBJ烏[Nʒu̎擾(3񕽋ϒl)
		while(1){
			if(m_positionIn.isNew()){
				m_positionIn.read();
								
				if((m_position.data[11]>m_cameraXLimitMin)&&(m_position.data[11]<m_cameraXLimitMax)&&(m_position.data[15]>m_cameraYLimitMin)&&(m_position.data[15]<m_cameraYLimitMax)){
					
					if(count>=4){
						CameraXPosition = CameraXPosition+m_position.data[11];
					}
					CameraXPositionBuf[count]=m_position.data[11];

					count++;
					if(count == 7){
						CameraXPosition = CameraXPosition/3;
						for(int i=4;i<7;i++){
							if(abs(CameraXPositionBuf[i]-CameraXPosition) > 10){
								std::cout<<"[NY΍ERROR"<<std::endl;
								RTC_ERROR(("[NY΍ERROR"));
								return RTC::RTC_ERROR;
							}
						}
						break;
					}
				}else{
					std::cout<<"[NF͈͊OERROR"<<std::endl;
					RTC_ERROR(("[NF͈͊OERROR"));
					return RTC::RTC_ERROR;
				}
			}
		}

		//[Nʒu̍W
		//{bgXW(JxW)
		m_Pos[2].carPos[0][3]=m_P[2][0]+m_cameraLengthX*(double)(CameraXPosition-m_cameraCalibrationX)/640.0;
		m_Pos[3].carPos[0][3]=m_P[3][0]+m_cameraLengthX*(double)(CameraXPosition-m_cameraCalibrationX)/640.0;

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P1,Mov)
		std::cout<<"{bg(P1,JMov)"<<std::endl;
		RTC_INFO(("{bg(P1,JMov)"));
		JARA_ARM::JointPos jnt;
		jnt.length(2);		
		jnt[0]=m_Pos[1].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[1].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		
		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P2,Mov)
		std::cout<<"{bg(P2,JMov)"<<std::endl;
		RTC_INFO(("{bg(P2,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[2].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[2].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P3,Mvs)
		std::cout<<"{bg(P3,JMov)"<<std::endl;
		RTC_INFO(("{bg(P3,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[3].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[3].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//nh(DIOo̓|[g[1],[2])
		std::cout<<"nhCLOSE"<<std::endl;
		RTC_INFO(("nhCLOSE"));
		m_digitalOutput.data.length(3);
		m_digitalOutput.data[1]=false;
		m_digitalOutput.data[2]=true;
		m_digitalOutputOut.write();
		while(1){
			if(m_digitalInputIn.isNew()){
				m_digitalInputIn.read();
				if(m_digitalInput.data[1] == true){
					break;
				}
			}
		}
		
		//{bg(P2,Mvs)
		std::cout<<"{bg(P2,JMov)"<<std::endl;
		RTC_INFO(("{bg(P2,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[2].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[2].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P4,Mvs)
		std::cout<<"{bg(P4,JMov)"<<std::endl;
		RTC_INFO(("{bg(P4,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[4].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[4].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P5,Mvs)
		std::cout<<"{bg(P5,JMov)"<<std::endl;
		RTC_INFO(("{bg(P5,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[5].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[5].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//nhJ(DIOo̓|[g[1],[2])
		std::cout<<"nhOPEN"<<std::endl;
		RTC_INFO(("nhOPEN"));
		m_digitalOutput.data.length(3);
		m_digitalOutput.data[1]=true;
		m_digitalOutput.data[2]=false;
		m_digitalOutputOut.write();
		while(1){
			if(m_digitalInputIn.isNew()){
				m_digitalInputIn.read();
				if(m_digitalInput.data[1] == true){
					break;
				}
			}
		}

		//{bg(P4,Mvs)
		std::cout<<"{bg(P4,JMov)"<<std::endl;
		RTC_INFO(("{bg(P4,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[4].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[4].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		do{
			m_ManipulatorCommonInterface_Common->getState(m_state);
			if(m_state&0x4){
				std::cout <<"A["<< std::endl;
				RTC_ERROR(("A["));
				return RTC::RTC_ERROR;
			}
		}while(m_state&0x2);

		//{bg(P1,Mov)
		std::cout<<"{bg(P1,JMov)"<<std::endl;
		RTC_INFO(("{bg(P1,JMov)"));
		jnt.length(2);		
		jnt[0]=m_Pos[1].carPos[0][3]/1000.0;//[mm]->[m]
		jnt[1]=m_Pos[1].carPos[1][3]/1000.0;//[mm]->[m]
		m_rid=m_ManipulatorCommonInterface_Middle->movePTPJointAbs(jnt);
		if(m_rid->id != 0){
			std::cout << std::endl<< m_rid->comment << std::endl;
			//ErrorJ
			return RTC::RTC_ERROR;
		}

		std::cout<<"JԂ:"<<++m_Repeat<<""<<std::endl<<"(c"<<m_cycleCount-m_Repeat<<")"<<std::endl;
		RTC_INFO(("JԂ:%d\n(c%d)",m_Repeat,m_cycleCount-m_Repeat));
	}

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t IntegratedControlRTC::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t IntegratedControlRTC::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t IntegratedControlRTC::onReset(RTC::UniqueId ec_id)
{
	std::cout<<"A[NA܂"<<std::endl;
	RTC_INFO(("A[NA܂"));

	//A[NA
	m_rid=m_ManipulatorCommonInterface_Common->clearAlarms();
	if(m_rid->id != 0){
		std::cout << std::endl<< m_rid->comment << std::endl;
		//ErrorJ
		return RTC::RTC_ERROR;
	}

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t IntegratedControlRTC::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t IntegratedControlRTC::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void IntegratedControlRTCInit(RTC::Manager* manager)
  {
    coil::Properties profile(integratedcontrolrtc_spec);
    manager->registerFactory(profile,
                             RTC::Create<IntegratedControlRTC>,
                             RTC::Delete<IntegratedControlRTC>);
  }
  
};


