// -*- C++ -*-
/*!
 * @file  RTM_ORiN_Converter_YAMAHA.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "RTM_ORiN_Converter_YAMAHA.h"
#include "defcommand.h"
#include "useORiN.h"

// Module specification
// <rtc-template block="module_spec">
static const char* rtm_orin_converter_yamaha_spec[] =
  {
    "implementation_id", "RTM_ORiN_Converter_YAMAHA",
    "type_name",         "RTM_ORiN_Converter_YAMAHA",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "Saitama Univ. Design Lab.",
    "category",          "Industrial Robot",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.Port", "com:1",
    "conf.default.Timeout", "50000",
    // Widget
    "conf.__widget__.Port", "text",
    "conf.__widget__.Timeout", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
RTM_ORiN_Converter_YAMAHA::RTM_ORiN_Converter_YAMAHA(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_ManipulatorCommonInterface_CommonPort("ManipulatorCommonInterface_Common"),
    m_ManipulatorCommonInterface_MiddlePort("ManipulatorCommonInterface_Middle")

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
RTM_ORiN_Converter_YAMAHA::~RTM_ORiN_Converter_YAMAHA()
{
}



RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onInitialize()
{
  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  
  // Set OutPort buffer
  
  // Set service provider to Ports
  m_ManipulatorCommonInterface_CommonPort.registerProvider("ManipulatorCommonInterface_Common", "JARA_ARM::ManipulatorCommonInterface_Common", m_ManipulatorCommonInterface_Common);
  m_ManipulatorCommonInterface_MiddlePort.registerProvider("ManipulatorCommonInterface_Middle", "JARA_ARM::ManipulatorCommonInterface_Middle", m_ManipulatorCommonInterface_Middle);
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  addPort(m_ManipulatorCommonInterface_CommonPort);
  addPort(m_ManipulatorCommonInterface_MiddlePort);
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("Port", m_Port, "com:1");
  bindParameter("Timeout", m_Timeout, "50000");
  
  // </rtc-template>
  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onActivated(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"           Activate           "<<std::endl<<"******************************"<<std::endl<<std::endl;

	//Configl̑
	timeout = m_Timeout;
	int len = m_Port.length();
	PortParam = new char[len+1];
	memcpy(PortParam, m_Port.c_str(), len+1);

	std::cout<<"Configuration"<<std::endl<<"ʐM|[gp[^"<<std::endl<<" PortParam : "<<PortParam<<std::endl<<"ʐM^CAEg"<<std::endl<<" Timeout : "<<timeout<<std::endl<<std::endl;

	std::cout<<std::endl<<"******************************"<<std::endl<<"           Execute            "<<std::endl<<"******************************"<<std::endl<<std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"          Deactivate          "<<std::endl<<"******************************"<<std::endl<<std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onExecute(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}


RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onAborting(RTC::UniqueId ec_id)
{
	std::cout<<std::endl<<"******************************"<<std::endl<<"            Error             "<<std::endl<<"******************************"<<std::endl<<std::endl;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t RTM_ORiN_Converter_YAMAHA::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void RTM_ORiN_Converter_YAMAHAInit(RTC::Manager* manager)
  {
    coil::Properties profile(rtm_orin_converter_yamaha_spec);
    manager->registerFactory(profile,
                             RTC::Create<RTM_ORiN_Converter_YAMAHA>,
                             RTC::Delete<RTM_ORiN_Converter_YAMAHA>);
  }
  
};


