// -*- C++ -*-
/*!
 * @file  TimedBooleanSeqOutRTC.cpp
 * @brief ModuleDescription
 * @date $Date$
 *
 * $Id$
 */

#include "TimedBooleanSeqOutRTC.h"

// Module specification
// <rtc-template block="module_spec">
static const char* timedbooleanseqoutrtc_spec[] =
  {
    "implementation_id", "TimedBooleanSeqOutRTC",
    "type_name",         "TimedBooleanSeqOutRTC",
    "description",       "ModuleDescription",
    "version",           "1.0.0",
    "vendor",            "Saitama Univ. Design Lab.",
    "category",          "EngineeringSample",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "1",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.decValue", "0",
    // Widget
    "conf.__widget__.decValue", "text",
    // Constraints
    ""
  };
// </rtc-template>

/*!
 * @brief constructor
 * @param manager Maneger Object
 */
TimedBooleanSeqOutRTC::TimedBooleanSeqOutRTC(RTC::Manager* manager)
    // <rtc-template block="initializer">
  : RTC::DataFlowComponentBase(manager),
    m_digitalInputIn("digitalInput", m_digitalInput),
    m_digitalOutputOut("digitalOutput", m_digitalOutput)

    // </rtc-template>
{
}

/*!
 * @brief destructor
 */
TimedBooleanSeqOutRTC::~TimedBooleanSeqOutRTC()
{
}



RTC::ReturnCode_t TimedBooleanSeqOutRTC::onInitialize()
{
	std::cout<<"Initialize"<<std::endl;

  // Registration: InPort/OutPort/Service
  // <rtc-template block="registration">
  // Set InPort buffers
  addInPort("digitalInput", m_digitalInputIn);
  
  // Set OutPort buffer
  addOutPort("digitalOutput", m_digitalOutputOut);
  
  // Set service provider to Ports
  
  // Set service consumers to Ports
  
  // Set CORBA Service Ports
  
  // </rtc-template>

  // <rtc-template block="bind_config">
  // Bind variables and configuration variable
  bindParameter("decValue", m_decValue, "0");
  
  // </rtc-template>
  m_digitalOutput.data.length(32);
  m_outportValueOld = 4294967295;

  std::cout<<"InitializeI"<<std::endl;

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t TimedBooleanSeqOutRTC::onFinalize()
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TimedBooleanSeqOutRTC::onStartup(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TimedBooleanSeqOutRTC::onShutdown(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/


RTC::ReturnCode_t TimedBooleanSeqOutRTC::onActivated(RTC::UniqueId ec_id)
{
	std::cout<<"Activate"<<std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t TimedBooleanSeqOutRTC::onDeactivated(RTC::UniqueId ec_id)
{
	std::cout<<"Deactivate"<<std::endl;

  return RTC::RTC_OK;
}


RTC::ReturnCode_t TimedBooleanSeqOutRTC::onExecute(RTC::UniqueId ec_id)
{
	if(m_digitalInputIn.isNew()){
		m_digitalInputIn.read();
		unsigned long inportValue = 0;
		for(int i=0;i<m_digitalInput.data.length();i++){
			if(m_digitalInput.data[i]){
				inportValue = inportValue + (unsigned long)(pow(2.0,i));
			}
		}
		std::cout << "(10i)inportValue =  " << inportValue << std::endl;
		std::cout.setf( std::ios::hex, std::ios::basefield );
		std::cout.setf( std::ios::showbase );
		std::cout << "(16i)inportValue =  " << inportValue << std::endl;
		std::cout.setf( std::ios::dec, std::ios::basefield );
		std::cout << "( 2i)inportValue =  " << std::bitset<32>((unsigned long long)inportValue) << std::endl;
		RTC_INFO(("( 2i)inportValue = %ld",std::bitset<32>((unsigned long long)inportValue)));
	}

	if(m_outportValueOld != m_decValue){
		unsigned long outportValue = m_decValue;
		m_outportValueOld = outportValue;
		std::cout << "(10i)outportValue = " << outportValue << std::endl;
		std::cout.setf( std::ios::hex, std::ios::basefield );
		std::cout.setf( std::ios::showbase );
		std::cout << "(16i)outportValue = " << outportValue << std::endl;
		std::cout.setf( std::ios::dec, std::ios::basefield );
		std::cout << "( 2i)outportValue = " << std::bitset<32>((unsigned long long)outportValue) << std::endl;
		RTC_INFO(("( 2i)outportValue = %ld",std::bitset<32>((unsigned long long)outportValue)));
		for(int i=m_digitalOutput.data.length();i>0;i--){
			if( outportValue >= (unsigned long)(pow(2.0,i-1)) ){
				outportValue = outportValue - (unsigned long)(pow(2.0,i-1));
				m_digitalOutput.data[i-1] = true;
			}else{
				m_digitalOutput.data[i-1] = false;
			}
		}
		m_digitalOutputOut.write();
	}

  return RTC::RTC_OK;
}

/*
RTC::ReturnCode_t TimedBooleanSeqOutRTC::onAborting(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TimedBooleanSeqOutRTC::onError(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TimedBooleanSeqOutRTC::onReset(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TimedBooleanSeqOutRTC::onStateUpdate(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/

/*
RTC::ReturnCode_t TimedBooleanSeqOutRTC::onRateChanged(RTC::UniqueId ec_id)
{
  return RTC::RTC_OK;
}
*/



extern "C"
{
 
  void TimedBooleanSeqOutRTCInit(RTC::Manager* manager)
  {
    coil::Properties profile(timedbooleanseqoutrtc_spec);
    manager->registerFactory(profile,
                             RTC::Create<TimedBooleanSeqOutRTC>,
                             RTC::Delete<TimedBooleanSeqOutRTC>);
  }
  
};


